@@ -1,4 +1,4 @@
-# $Id: Protocol.pm,v 1.30 1997/12/15 20:29:10 aas Exp $
+# $Id: Protocol.pm,v 1.37 2001/04/19 05:34:03 gisle Exp $
 
 package LWP::Protocol;
 
@@ -38,16 +38,18 @@ The following methods and functions are provided:
 
 require LWP::MemberMixin;
 @ISA = qw(LWP::MemberMixin);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.37 $ =~ /(\d+)\.(\d+)/);
 
 use strict;
 use Carp ();
-use HTTP::Status 'RC_INTERNAL_SERVER_ERROR';
+use HTTP::Status ();
+use HTTP::Response;
 require HTML::HeadParser;
 
 my %ImplementedBy = (); # scheme => classname
 
 
-=item $prot = new HTTP::Protocol;
+=item $prot = LWP::Protocol->new()
 
 The LWP::Protocol constructor is inherited by subclasses. As this is a
 virtual base class this method should B<not> be called directly.
@@ -56,17 +58,22 @@ virtual base class this method should B<not> be called directly.
 
 sub new
 {
-    my($class) = @_;
+    my($class, $scheme, $ua) = @_;
 
     my $self = bless {
-	'timeout' => 0,
-	'parse_head' => 1,
+	scheme => $scheme,
+	ua => $ua,
+
+	# historical/redundant
+        parse_head => $ua->{parse_head},
+        max_size => $ua->{max_size},
     }, $class;
+
     $self;
 }
 
 
-=item $prot = LWP::Protocol::create($url)
+=item $prot = LWP::Protocol::create($scheme)
 
 Create an object of the class implementing the protocol to handle the
 given scheme. This is a function, not a method. It is more an object
@@ -77,12 +84,14 @@ use to access protocols.
 
 sub create
 {
-    my $scheme = shift;
+    my($scheme, $ua) = @_;
     my $impclass = LWP::Protocol::implementor($scheme) or
 	Carp::croak("Protocol scheme '$scheme' is not supported");
 
     # hand-off to scheme specific implementation sub-class
-    return $impclass->new($scheme);
+    my $protocol = $impclass->new($scheme, $ua);
+
+    return $protocol;
 }
 
 
@@ -112,11 +121,11 @@ sub implementor
     $ic = "LWP::Protocol::nntp" if $scheme eq 'news'; #XXX ugly hack
     no strict 'refs';
     # check we actually have one for the scheme:
-    unless (defined @{"${ic}::ISA"}) {
+    unless (@{"${ic}::ISA"}) {
 	# try to autoload it
 	eval "require $ic";
 	if ($@) {
-	    if ($@ =~ /^Can't locate/) { #' #emacs get confused by '
+	    if ($@ =~ /Can't locate/) { #' #emacs get confused by '
 		$ic = '';
 	    } else {
 		die "$@\n";
@@ -147,18 +156,7 @@ sub request
 }
 
 
-=item $prot->timeout($seconds)
-
-Get and set the timeout value in seconds
-
-
-=item $prot->parse_head($yesno)
-
-Should we initialize response headers from the <head> section of HTML
-documents.
-
-=cut
-
+# legacy
 sub timeout    { shift->_elem('timeout',    @_); }
 sub parse_head { shift->_elem('parse_head', @_); }
 sub max_size   { shift->_elem('max_size',   @_); }
@@ -191,8 +189,7 @@ sub collect
 {
     my ($self, $arg, $response, $collector) = @_;
     my $content;
-    my($parse_head, $timeout, $max_size) =
-      @{$self}{qw(parse_head timeout max_size)};
+    my($parse_head, $max_size) = @{$self}{qw(parse_head max_size)};
 
     my $parser;
     if ($parse_head && $response->content_type eq 'text/html') {
@@ -220,8 +217,8 @@ sub collect
     elsif (!ref($arg)) {
 	# filename
 	open(OUT, ">$arg") or
-	    return new HTTP::Response RC_INTERNAL_SERVER_ERROR,
-			  "Cannot write to '$arg': $!";
+	    return HTTP::Response->new(&HTTP::Status::RC_INTERNAL_SERVER_ERROR,
+			  "Cannot write to '$arg': $!");
         binmode(OUT);
         local($\) = ""; # ensure standard $OUTPUT_RECORD_SEPARATOR
 	while ($content = &$collector, length $$content) {
@@ -258,8 +255,8 @@ sub collect
 	}
     }
     else {
-	return new HTTP::Response RC_INTERNAL_SERVER_ERROR,
-				  "Unexpected collect argument  '$arg'";
+	return HTTP::Response->new(&HTTP::Status::RC_INTERNAL_SERVER_ERROR,
+				  "Unexpected collect argument  '$arg'");
     }
     $response;
 }
@@ -294,7 +291,7 @@ for examples of usage.
 
 =head1 COPYRIGHT
 
-Copyright 1995-1997 Gisle Aas.
+Copyright 1995-2001 Gisle Aas.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
