@@ -1,4 +1,4 @@
-# $Id: RobotRules.pm,v 1.16.2.2 1998/09/11 12:42:04 aas Exp $
+# $Id: RobotRules.pm,v 1.21 2000/04/07 20:17:54 gisle Exp $
 
 package WWW::RobotRules;
 
@@ -30,15 +30,15 @@ WWW::RobotsRules - Parse robots.txt files
 
 =head1 DESCRIPTION
 
-This module parses a F<robots.txt> file as specified in
+This module parses a F</robots.txt> file as specified in
 "A Standard for Robot Exclusion", described in
 <http://info.webcrawler.com/mak/projects/robots/norobots.html>
-Webmasters can use the F<robots.txt> file to disallow conforming
-robots access to parts of their WWW server.
+Webmasters can use the F</robots.txt> file to disallow conforming
+robots access to parts of their web site.
 
 The parsed file is kept in the WWW::RobotRules object, and this object
-provide methods to check if access to a given URL is prohibited.  The
-same WWW::RobotRules object can parse multiple F<robots.txt> files.
+provides methods to check if access to a given URL is prohibited.  The
+same WWW::RobotRules object can parse multiple F</robots.txt> files.
 
 The following methods are provided:
 
@@ -46,14 +46,14 @@ The following methods are provided:
 
 =cut
 
-$VERSION = sprintf("%d.%02d", q$Revision: 1.16.2.2 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.21 $ =~ /(\d+)\.(\d+)/);
 sub Version { $VERSION; }
 
 use strict;
 use URI ();
 
 
-=item $rules = new WWW::RobotRules 'MOMspider/1.0'
+=item $rules = WWW::RobotRules->new($robot_name)
 
 This is the constructor for WWW::RobotRules objects.  The first 
 argument given to new() is the name of the robot. 
@@ -73,7 +73,7 @@ sub new {
 }
 
 
-=item $rules->parse($url, $content, $fresh_until)
+=item $rules->parse($robot_txt_url, $content, $fresh_until)
 
 The parse() method takes as arguments the URL that was used to
 retrieve the F</robots.txt> file, and the contents of the file.
@@ -81,9 +81,9 @@ retrieve the F</robots.txt> file, and the contents of the file.
 =cut
 
 sub parse {
-    my($self, $url, $txt, $fresh_until) = @_;
-    $url = URI->new("$url");
-    my $netloc = $url->authority;
+    my($self, $robot_txt_uri, $txt, $fresh_until) = @_;
+    $robot_txt_uri = URI->new("$robot_txt_uri");
+    my $netloc = $robot_txt_uri->authority;
 
     $self->clear_rules($netloc);
     $self->fresh_until($netloc, $fresh_until || (time + 365*24*3600));
@@ -133,7 +133,17 @@ sub parse {
 	    my $disallow = $1;
 	    $disallow =~ s/\s+$//;
 	    if (length $disallow) {
-		$disallow = URI->new($disallow, $url)->path_query;
+		my $ignore;
+		eval {
+		    my $u = URI->new_abs($disallow, $robot_txt_uri);
+		    $ignore++ if $u->scheme ne $robot_txt_uri->scheme;
+		    $ignore++ if lc($u->host) ne lc($robot_txt_uri->host);
+		    $ignore++ if $u->port ne $robot_txt_uri->port;
+		    $disallow = $u->path_query;
+		    $disallow = "/" unless length $disallow;
+		};
+		next if $@;
+		next if $ignore;
 	    }
 
 	    if ($is_me) {
@@ -166,21 +176,21 @@ sub is_me {
     return index(lc($ua), lc($me)) >= 0;
 }
 
-=item $rules->allowed($url)
+=item $rules->allowed($uri)
 
 Returns TRUE if this robot is allowed to retrieve this URL.
 
 =cut
 
 sub allowed {
-    my($self, $url) = @_;
-    $url = URI->new("$url");
-    my $netloc = $url->authority;
+    my($self, $uri) = @_;
+    $uri = URI->new("$uri");
+    my $netloc = $uri->authority;
 
     my $fresh_until = $self->fresh_until($netloc);
     return -1 if !defined($fresh_until) || $fresh_until < time;
 
-    my $str = $url->path_query;
+    my $str = $uri->path_query;
     my $rule;
     for $rule ($self->rules($netloc)) {
 	return 1 unless length $rule;
@@ -227,7 +237,6 @@ sub visit {
     my($self, $netloc, $time) = @_;
     $time ||= time;
     $self->{'loc'}{$netloc}{'last'} = $time;
-    
     my $count = \$self->{'loc'}{$netloc}{'count'};
     if (!defined $$count) {
 	$$count = 1;
@@ -284,7 +293,6 @@ sub dump
     for (keys %{$self->{'loc'}}) {
 	my @rules = $self->rules($_);
 	print "$_: ", join("; ", @rules), "\n";
-	
     }
 }
 
